@@ -1,7 +1,7 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/ce_parser.c,v 1.6 2005/12/19 15:46:57 richb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/ce_parser.c,v 1.8 2006/09/13 18:03:53 sampie Exp $
  *
- *  Copyright (C) 2004-2005 Sami Pietila
+ *  Copyright (C) 2004-2006 Sami Pietila
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -40,6 +40,7 @@ ce_parse_(char *expression, int result[MP_SIZE], int flags)
     if (strlen(expression)) {
         parser_state.i = 0;
         parser_state.buff = gc_strdup(expression);
+		v->math_error = 0;
         ret = ceparse();
         free(parser_state.buff);
     }
@@ -52,11 +53,16 @@ ce_parse_(char *expression, int result[MP_SIZE], int flags)
         if ((flags & ANS) != (parser_state.flags & ANS)) {
             return -EINVAL;
         }
+
+		if (v->math_error) {
+		  return v->math_error;
+		}
+
         if (flags & ANS) {
             memcpy(result, parser_state.ret, sizeof(int)*MP_SIZE);
         }
 
-        return(0);
+		return 0;
     }
 }
 
