@@ -1,5 +1,5 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/functions.c,v 1.82 2006/08/09 16:54:01 richb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/functions.c,v 1.89 2006/12/15 15:27:37 richb Exp $
  *
  *  Copyright (c) 1987-2006 Sun Microsystems, Inc. All Rights Reserved.
  *           
@@ -143,13 +143,6 @@ new_state(void)
 }
 
 
-int
-is_undo_step()
-{
-    return(v->h.current != v->h.begin);
-}
-
-
 void
 perform_undo(void)
 {
@@ -162,6 +155,12 @@ perform_undo(void)
     update_undo_redo_button_sensitivity();
 }
 
+int
+is_undo_step()
+{
+    return(v->h.current != v->h.begin);
+}
+
 void
 perform_redo(void)
 {
@@ -242,7 +241,7 @@ exp_append(char *text)
             assert(0);
         }
     } else {
-        strcpy(buf, text);
+        STRCPY(buf, text);
     }
     free(e->expression);
     e->expression = buf;
@@ -258,7 +257,7 @@ exp_del()
 }
 
 
-static int 
+int 
 usable_num(int MPnum[MP_SIZE]) 
 {
     int ret = 0;
@@ -269,7 +268,6 @@ usable_num(int MPnum[MP_SIZE])
         ret = ce_parse(e->expression, MPnum);
     } else {
 	do_zero(MPnum);
-        //mpstr(v->MPresult, MPnum);
     }
 
     return ret;
@@ -408,7 +406,7 @@ str_replace(char **str, char *from, char *to)
             memcpy(postfix, *str+i+3, len-j);
 
             print = malloc(strlen(to)+i+len-j+1);
-            sprintf(print, "%s%s%s", prefix, to, postfix);
+            SPRINTF(print, "%s%s%s", prefix, to, postfix);
             free(prefix);
             free(postfix);
             free(*str);
@@ -495,10 +493,12 @@ do_expression()
         int i = char_val(e->button.value[0]);
         char reg[3];
         int n = '0' +  i;
+
         snprintf(reg, 3, "R%c", n);
         exp_append(reg);
     } else if (e->button.flags & con) {
         int *MPval = v->MPcon_vals[char_val(e->button.value[0])];
+
         exp_append(make_number(MPval, v->base, FALSE));
     } else if (e->button.flags & bsp) {
         if (exp_has_postfix(e->expression, "Ans")) { 
@@ -511,38 +511,50 @@ do_expression()
     } else if (e->button.flags & inv) {
         exp_inv();
     } else if (e->button.flags & enter) {
-        if (e->expression) {
+	  if (e->expression) {
 	    if (strcmp(e->expression, "Ans")) {
-		    int MPval[MP_SIZE];
-		    int ret = ce_parse(e->expression, MPval);
-		    if (!ret) {
+		  int MPval[MP_SIZE];
+		  int ret = ce_parse(e->expression, MPval);
+
+		  if (!ret) {
 			mpstr(MPval, e->ans);
 			exp_replace("Ans");
-		    } else {
+		  } else {
 			char *message = NULL;
+
 			switch (ret) {
-			case -PARSER_ERR_INVALID_BASE:
-			    message = _("Invalid number for the current base");
-			    break;
-			case -PARSER_ERR_TOO_LONG_NUMBER:
-			    message = _("Too long number");
-			    break;
-			case -PARSER_ERR_BITWISEOP:
-			    message = _("Invalid bitwise operation parameter(s)");
-			    break;
-			default:
-			    message = _("Malformed expression");
-			    break;
+			    case -PARSER_ERR_INVALID_BASE:
+			        message = _("Invalid number for the current base");
+			        break;
+
+			    case -PARSER_ERR_TOO_LONG_NUMBER:
+			        message = _("Too long number");
+			        break;
+
+			    case -PARSER_ERR_BITWISEOP:
+			        message = _("Invalid bitwise operation parameter(s)");
+			        break;
+
+			    case -PARSER_ERR_MODULUSOP:
+			        message = _("Invalid modulus operation parameter(s)");
+			        break;
+
+			    case -MPMATH_ERR:
+			        message = _("Math operation error");
+			        break;
+
+			    default:
+			        message = _("Malformed expression");
 			}
 			update_statusbar(message, "gtk-dialog-error");
-		    }
+		  }
 	    } else {
-                perform_undo();
-                if (is_undo_step()) {
-                    perform_undo();
-                }
+		  perform_undo();
+		  if (is_undo_step()) {
+			perform_undo();
+		  }
 	    }
-	}
+	  }
     } else {
 	exp_append(btext);
 	if (e->button.flags & func) {
@@ -610,11 +622,6 @@ do_calc()      /* Perform arithmetic calculation and display result. */
         mpmul(MP1, v->MPdisp_val, MP2);
         mpsub(v->MPresult, MP2, v->MPresult);
 
-    } else if (IS_KEY(v->cur_op, KEY_PER.value[0])) {     /* % */
-        mpmul(v->MPresult, v->MPdisp_val, v->MPresult);
-        MPstr_to_num("0.01", DEC, MP1);
-        mpmul(v->MPresult, MP1, v->MPresult);
-
     } else if (IS_KEY(v->cur_op, KEY_YTOX.value[0])) {    /* y^x */
         calc_xpowy(v->MPresult, v->MPdisp_val, v->MPresult);
 
@@ -697,10 +704,18 @@ do_trig()
 }
 
 
+void
+do_percent()
+{
+    calc_percent(v->MPdisp_val, v->MPresult);
+    show_display(v->MPresult);
+    mpstr(v->MPresult, v->MPdisp_val);
+}
+
 int
 do_tfunc(int s[MP_SIZE], int t[MP_SIZE], enum trig_func tfunc)
 {
-    // Assumes the SIN=0, COS=1, TAN=2
+    /* Assumes the SIN=0, COS=1, TAN=2. */
     
     enum trigfunc_type conv_table[3][4] = {
 	{sin_t, asin_t, sinh_t, asinh_t},
@@ -714,11 +729,13 @@ do_tfunc(int s[MP_SIZE], int t[MP_SIZE], enum trig_func tfunc)
 
     assert(tfunc < 3); 
     
-    if (!v->current) return -EINVAL;
+    if (!v->current) {
+        return -EINVAL;
+    }
     
     calc_trigfunc(conv_table[tfunc][mode], s, t);
     
-    return 0;
+    return(0);
 }
 
 
@@ -784,18 +801,20 @@ do_constant()
     assert(v->current->value[0] <= '9');
     
     switch (v->syntax) {
-    case npa: {
-	int *MPval = v->MPcon_vals[char_val(v->current->value[0])];
-	mpstr(MPval, v->MPdisp_val);
-	break;
-    }
-	
-    case exprs: {
-	struct exprm_state *e = get_state();
-	e->button.flags = con;
-	do_expression();
-    }
-            break;
+        case npa: {
+	    int *MPval = v->MPcon_vals[char_val(v->current->value[0])];
+
+	    mpstr(MPval, v->MPdisp_val);
+	    break;
+        }
+
+        case exprs: {
+	    struct exprm_state *e = get_state();
+
+	    e->button.flags = con;
+	    do_expression();
+        }
+        break;
 
         default:
             assert(0);
@@ -845,38 +864,36 @@ do_exchange()         /* Exchange display with memory register. */
     int MPexpr[MP_SIZE];
 
     switch (v->syntax) {
-        case npa:
-	  {
-	    mpstr(v->MPdisp_val, MPtemp);
-	    mpstr(v->MPmvals[char_val(v->current->value[0])], v->MPdisp_val);
-	    mpstr(MPtemp, v->MPmvals[char_val(v->current->value[0])]);
-	    make_registers();
-	  }
-	  break;
+        case npa: {
+            mpstr(v->MPdisp_val, MPtemp);
+            mpstr(v->MPmvals[char_val(v->current->value[0])], v->MPdisp_val);
+            mpstr(MPtemp, v->MPmvals[char_val(v->current->value[0])]);
+            make_registers();
+        }
+        break;
 
-        case exprs:
-	  {
-	      struct exprm_state *e = get_state();
-	      int ret = usable_num(MPexpr);
-	      int n = char_val(e->button.value[0]);
-	      if (ret) {
-		  update_statusbar(_("No sane value to store"), 
-				   "gtk-dialog-error");
-	      } else {
-		  mpstr(v->MPmvals[n], MPtemp);
-		  mpstr(MPexpr, v->MPmvals[n]);
-		  mpstr(MPtemp, e->ans);	      
-		  exp_replace("Ans");
-		  refresh_display();
-		  make_registers();
+        case exprs: {
+              struct exprm_state *e = get_state();
+              int ret = usable_num(MPexpr);
+              int n = char_val(e->button.value[0]);
+
+              if (ret) {
+                  update_statusbar(_("No sane value to store"), 
+                                   "gtk-dialog-error");
+              } else {
+                  mpstr(v->MPmvals[n], MPtemp);
+                  mpstr(MPexpr, v->MPmvals[n]);
+                  mpstr(MPtemp, e->ans);	      
+                  exp_replace("Ans");
+                  refresh_display();
+                  make_registers();
 	      }
 	  }
-	  break;
+          break;
 
         default:
             assert(0);
     }
-
 }
 
 
@@ -914,7 +931,7 @@ do_factorial(int *MPval, int *MPres)
  *        then we've overflowed. This is to provide the same look&feel
  *        as V3.
  *
- *  XXX:  Needs to be imtproved. Shouldn't need to convert to a double in
+ *  XXX:  Needs to be improved. Shouldn't need to convert to a double in
  *        order to check this.
  */
 
@@ -958,31 +975,31 @@ do_function()      /* Perform a user defined function. */
 
     switch (v->syntax) {
         case npa:
-		    assert(v->current->value[0] >= '0');
-			assert(v->current->value[0] <= '9');
-		  
-			fno = char_val(v->current->value[0]);
-			ret = 0;
-			str = v->fun_vals[fno];
-			assert(str);
+            assert(v->current->value[0] >= '0');
+            assert(v->current->value[0] <= '9');
+
+            fno = char_val(v->current->value[0]);
+            ret = 0;
+            str = v->fun_vals[fno];
+            assert(str);
 
             ret = lr_udf_parse(str);
             break;
 
-	    case exprs: {
-		    struct exprm_state *e = get_state();
+        case exprs: {
+            struct exprm_state *e = get_state();
 
-		    assert(e->button.value[0] >= '0');
-			assert(e->button.value[0] <= '9');
-		  
-			fno = char_val(e->button.value[0]);
-			ret = 0;
-			str = v->fun_vals[fno];
-			assert(str);
+            assert(e->button.value[0] >= '0');
+            assert(e->button.value[0] <= '9');
+
+            fno = char_val(e->button.value[0]);
+            ret = 0;
+            str = v->fun_vals[fno];
+            assert(str);
 
             ret = ce_udf_parse(str);
-	    }
-            break;
+        }
+        break;
 
         default:
             assert(0);
@@ -1107,12 +1124,12 @@ do_number()
     }
 
     if (v->toclear) {
-        SPRINTF(v->display, "%c", nextchar);
+        SNPRINTF(v->display, MAXLINE, "%c", nextchar);
         v->toclear = 0;
     } else {
         len = strlen(v->display);
         if (len < MAX_DIGITS) {
-	    SPRINTF(v->display+len, "%c", nextchar);
+	    SNPRINTF(v->display+len, MAXLINE, "%c", nextchar);
         }
     }
     set_display(v->display, TRUE);
@@ -1252,23 +1269,23 @@ static void
 do_rcl()
 {
     switch (v->syntax) {
-    case npa: {
-	int i = char_val(v->current->value[0]);
-	
-	mpstr(v->MPmvals[i], v->MPdisp_val);
-	break;
-    }
-	
-    case exprs: {
-	
-	struct exprm_state *e = get_state();
-	e->button.flags = regrcl;
-	do_expression();
-    }
-	break;
-	
-    default:
-	assert(0);
+        case npa: {
+            int i = char_val(v->current->value[0]);
+
+            mpstr(v->MPmvals[i], v->MPdisp_val);
+            break;
+        }
+
+        case exprs: {
+            struct exprm_state *e = get_state();
+
+            e->button.flags = regrcl;
+            do_expression();
+        }
+        break;
+
+        default:
+            assert(0);
     }
 }
 
@@ -1447,8 +1464,8 @@ do_shift()     /* Perform bitwise shift on display value. */
             int n = char_val(e->button.value[0]);
             int ret = usable_num(MPval);
 
-            if (ret) {
-	        update_statusbar(_("No sane value to store"), 
+            if (ret || !is_integer(MPval)) {
+	        update_statusbar(_("No sane value to do bitwise shift"), 
                                  "gtk-dialog-error");
 	        return;
             } 
@@ -1495,7 +1512,7 @@ push_num(int *MPval)        /* Try to push value onto the numeric stack. */
         return;
     }
     if (v->numsptr >= MAXSTACK) {
-        STRCPY(v->display, _("Numeric stack error"));
+        STRNCPY(v->display, _("Numeric stack error"), MAXLINE - 1);
         set_error_state(TRUE);
         set_display(v->display, FALSE);
         beep();
@@ -1516,7 +1533,7 @@ push_op(int val)     /* Try to push value onto the operand stack. */
         return;
     }
     if (v->opsptr >= MAXSTACK) {
-        STRCPY(v->display, _("Operand stack error"));
+        STRNCPY(v->display, _("Operand stack error"), MAXLINE - 1);
         set_error_state(TRUE);
         set_display(v->display, FALSE);
     } else {
@@ -1539,9 +1556,10 @@ set_main_title(enum mode_type modetype)  /* Set the title for main window. */
     char title[MAXLINE];
 
     if (modetype == BASIC) {
-        STRCPY(title, v->tool_label);
+        STRNCPY(title, v->tool_label, MAXLINE - 1);
     } else {
-        SPRINTF(title, "%s - %s", v->tool_label, _(mstrs[(int) modetype]));
+        SNPRINTF(title, MAXLINE, "%s - %s", v->tool_label, 
+                 _(mstrs[(int) modetype]));
     }
     set_title(FCP_KEY, title);
 }
