@@ -1,5 +1,5 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/get.c,v 1.46 2006/09/22 15:22:20 richb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/get.c,v 1.48 2006/10/30 18:12:46 richb Exp $
  *
  *  Copyright (c) 1987-2006 Sun Microsystems, Inc. All Rights Reserved.
  *           
@@ -72,7 +72,7 @@ get_bool_resource(enum res_type rtype, int *boolval)
         g_free(val);
         return(0);
     }
-    STRCPY(tempstr, val);
+    STRNCPY(tempstr, val, MAXLINE - 1);
     g_free(val);
     len = strlen(tempstr);
     for (n = 0; n < len; n++) {
@@ -176,7 +176,7 @@ static void
 getparam(char *s, char *argv[], char *errmes)
 {
     if (*argv != NULL && argv[0][0] != '-') {
-        STRCPY(s, *argv);
+        STRNCPY(s, *argv, MAXLINE - 1);
     } else { 
         FPRINTF(stderr, _("%s: %s as next argument.\n"), v->progname, errmes);
         exit(1);                        
@@ -196,7 +196,7 @@ get_str_resource(enum res_type rtype, char *strval)
         g_free(val);
         return(0);
     }
-    STRCPY(strval, val);
+    STRNCPY(strval, val, MAXLINE - 1);
     g_free(val);
     len = strlen(strval);
     for (i = 0; i < len; i++) {
@@ -270,7 +270,7 @@ init_vars()    /* Setup default values for various variables. */
     init_constant(0, "0.621");                 /* kms/hr <=> miles/hr. */
     init_constant(1, "1.4142135623");          /* square root of 2 */
     init_constant(2, "2.7182818284");          /* e */
-    init_constant(3, "3.1415926535");          /* pi */
+    init_constant(3, "3.1415926536");          /* pi */
     init_constant(4, "0.3937007");             /* cms <=> inch. */
     init_constant(5, "57.295779513");          /* degrees/radian. */
     init_constant(6, "1048576.0");             /* 2 ^ 20. */
@@ -382,38 +382,37 @@ read_resources()    /* Read all possible resources from the database. */
     v->syntax = exprs;
 
     if (get_str_resource(R_SYNTAX, str)) {
-      for (i = 0; i < MAXSYNTAX; i++) {
-		if (EQUAL(str, Rsstr[i])) {
-		  break;
-		}
-      }
-      
-      if (i == MAXSYNTAX) {
-		FPRINTF(stderr, _("%s: invalid syntax mode [%s]\n"), 
-				v->progname, str);
-      } else {
-		v->syntax = i;
-      }
+        for (i = 0; i < MAXSYNTAX; i++) {
+            if (EQUAL(str, Rsstr[i])) {
+                break;
+            }
+        }
+
+        if (i == MAXSYNTAX) {
+            FPRINTF(stderr, _("%s: invalid syntax mode [%s]\n"), 
+                    v->progname, str);
+        } else {
+            v->syntax = i;
+        }
     }
-	
-	v->bitcalculating_mode = 0; // default = off
+
+    v->bitcalculating_mode = 0;            /* default = off. */
 
     if (get_str_resource(R_BITCALC, str)) {
-      for (i = 0; i < MAXBITCALC; i++) {
-		if (EQUAL(str, Rcstr[i])) {
-		  break;
-		}
-      }
+        for (i = 0; i < MAXBITCALC; i++) {
+            if (EQUAL(str, Rcstr[i])) {
+                break;
+            }
+        }
       
-      if (i == MAXBITCALC) {
-		FPRINTF(stderr, _("%s: invalid bitcalc choice [%s]\n"), 
-				v->progname, str);
-      } else {
-		v->bitcalculating_mode = i;
-      }
+        if (i == MAXBITCALC) {
+            FPRINTF(stderr, _("%s: invalid bitcalc choice [%s]\n"), 
+                    v->progname, str);
+        } else {
+            v->bitcalculating_mode = i;
+        }
     }
 
-
     if (get_bool_resource(R_REGS, &boolval)) {
         v->rstate = boolval;
     }
@@ -425,7 +424,6 @@ read_resources()    /* Read all possible resources from the database. */
     if (get_bool_resource(R_TSEP, &boolval)) {
         v->show_tsep = boolval;
     }
-
 }
 
 
