@@ -1,7 +1,7 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/parser.h,v 1.6 2005/09/12 16:58:10 richb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/parser.h,v 1.9 2006/12/08 15:54:43 richb Exp $
  *
- *  Copyright (C) 2004-2005 Sami Pietila
+ *  Copyright (C) 2004-2006 Sami Pietila
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,8 +24,10 @@
 
 #define ANS 1
 
-#define PARSER_ERR_INVALID_BASE		10000
-#define PARSER_ERR_TOO_LONG_NUMBER 	10001
+#define PARSER_ERR_INVALID_BASE			10000
+#define PARSER_ERR_TOO_LONG_NUMBER 		10001
+#define PARSER_ERR_BITWISEOP		    	10002
+#define PARSER_ERR_MODULUSOP		    	10003
 
 struct parser_state {
     int flags;
